@@ -1,6 +1,6 @@
 /* events.c
 
-Copyright (C) 1999,2000 Tom Gilbert.
+Copyright (C) 1999-2003 Tom Gilbert.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -24,7 +24,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
 #include "feh.h"
-#include "feh_list.h"
 #include "filelist.h"
 #include "winwidget.h"
 #include "timers.h"
@@ -34,7 +33,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 feh_event_handler *ev_handler[LASTEvent];
 
-static void feh_event_handle_KeyPress(XEvent * ev);
 static void feh_event_handle_ButtonPress(XEvent * ev);
 static void feh_event_handle_ButtonRelease(XEvent * ev);
 static void feh_event_handle_ConfigureNotify(XEvent * ev);
@@ -52,7 +50,7 @@ feh_event_init(void)
    for (i = 0; i < LASTEvent; i++)
       ev_handler[i] = NULL;
 
-   ev_handler[KeyPress] = feh_event_handle_KeyPress;
+   ev_handler[KeyPress] = feh_event_handle_keypress;
    ev_handler[ButtonPress] = feh_event_handle_ButtonPress;
    ev_handler[ButtonRelease] = feh_event_handle_ButtonRelease;
    ev_handler[ConfigureNotify] = feh_event_handle_ConfigureNotify;
@@ -64,80 +62,44 @@ feh_event_init(void)
    D_RETURN_(4);
 }
 
-static void
-feh_event_handle_KeyPress(XEvent * ev)
-{
-   D_ENTER(4);
-   while (XCheckTypedWindowEvent(disp, ev->xkey.window, KeyPress, ev));
-   handle_keypress_event(ev, ev->xkey.window);
-   D_RETURN_(4);
-}
-
 static void
 feh_event_handle_ButtonPress(XEvent * ev)
 {
    winwidget winwid = NULL;
+   int scr_width, scr_height;
 
    D_ENTER(4);
-   /* hide the menus and get the heck out if it's a mouse-click on the
-      cover */
-   if (ev->xbutton.window == menu_cover)
-   {
-      feh_menu_hide(menu_root, True);
+
+   /* get the heck out if it's a mouse-click on the
+      cover, we'll hide the menus on release */
+   if (ev->xbutton.window == menu_cover) {
       D_RETURN_(4);
    }
 
-   if (!opt.no_menus
-       && ((ev->xbutton.button == opt.menu_button) || (opt.menu_button == 0))
-       && (((!opt.menu_ctrl_mask) && (!(ev->xbutton.state & ControlMask)))
-           || ((ev->xbutton.state & ControlMask) && (opt.menu_ctrl_mask))))
-   {
-      D(3, ("Menu Button Press event\n"));
-      winwid = winwidget_get_from_window(ev->xbutton.window);
-      if (winwid != NULL)
-      {
-         int x, y, b;
-         unsigned int c;
-         Window r;
+   scr_width = scr->width;
+   scr_height = scr->height;
+#ifdef HAVE_LIBXINERAMA
+    if (opt.xinerama && xinerama_screens) {
+      scr_width = xinerama_screens[xinerama_screen].width;
+      scr_height = xinerama_screens[xinerama_screen].height;
+    }
+#endif /* HAVE_LIBXINERAMA */
 
-         XQueryPointer(disp, winwid->win, &r, &r, &x, &y, &b, &b, &c);
-         if (winwid->type == WIN_TYPE_ABOUT)
-         {
-            if (!menu_about_win)
-               feh_menu_init_about_win();
-            feh_menu_show_at_xy(menu_about_win, winwid, x, y);
-         }
-         else if (winwid->type == WIN_TYPE_SINGLE)
-         {
-            if (!menu_single_win)
-               feh_menu_init_single_win();
-            feh_menu_show_at_xy(menu_single_win, winwid, x, y);
-         }
-         else if (winwid->type == WIN_TYPE_THUMBNAIL)
-         {
-            if (!menu_thumbnail_win)
-               feh_menu_init_thumbnail_win();
-            feh_menu_show_at_xy(menu_thumbnail_win, winwid, x, y);
-         }
-         else if (winwid->type == WIN_TYPE_THUMBNAIL_VIEWER)
-         {
-            if (!menu_single_win)
-               feh_menu_init_thumbnail_viewer();
-            feh_menu_show_at_xy(menu_thumbnail_viewer, winwid, x, y);
-         }
-         else
-         {
-            if (!menu_main)
-               feh_menu_init_main();
-            feh_menu_show_at_xy(menu_main, winwid, x, y);
-         }
+   winwid = winwidget_get_from_window(ev->xbutton.window);
+   if (winwid && winwid->caption_entry) {
+     D_RETURN_(4);
+   }
+
+   if (!opt.no_menus && EV_IS_MENU_BUTTON(ev)) {
+      D(3, ("Menu Button Press event\n"));
+      if (winwid != NULL) {
+        winwidget_show_menu(winwid);
       }
    }
    else if ((ev->xbutton.button == opt.rotate_button)
             && ((opt.no_rotate_ctrl_mask)
                 || (ev->xbutton.state & ControlMask)))
    {
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if (winwid != NULL)
       {
          opt.mode = MODE_ROTATE;
@@ -148,7 +110,6 @@ feh_event_handle_ButtonPress(XEvent * ev)
    else if ((ev->xbutton.button == opt.blur_button)
             && ((opt.no_blur_ctrl_mask) || (ev->xbutton.state & ControlMask)))
    {
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if (winwid != NULL)
       {
          opt.mode = MODE_BLUR;
@@ -159,7 +120,6 @@ feh_event_handle_ButtonPress(XEvent * ev)
    else if (ev->xbutton.button == opt.next_button)
    {
       D(3, ("Next Button Press event\n"));
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if (winwid != NULL)
       {
          D(3, ("Next button, but could be pan mode\n"));
@@ -173,7 +133,6 @@ feh_event_handle_ButtonPress(XEvent * ev)
    else if (ev->xbutton.button == opt.zoom_button)
    {
       D(3, ("Zoom Button Press event\n"));
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if (winwid != NULL)
       {
          D(3, ("Zoom mode baby!\n"));
@@ -187,8 +146,8 @@ feh_event_handle_ButtonPress(XEvent * ev)
          winwid->zoom = 1.0;
          if (winwid->full_screen)
          {
-            winwid->im_x = (scr->width - winwid->im_w) >> 1;
-            winwid->im_y = (scr->height - winwid->im_h) >> 1;
+            winwid->im_x = (scr_width - winwid->im_w) >> 1;
+            winwid->im_y = (scr_height - winwid->im_h) >> 1;
          }
          else if (opt.geom)
          {
@@ -224,21 +183,18 @@ feh_event_handle_ButtonPress(XEvent * ev)
    else if (ev->xbutton.button == opt.reload_button)
    {
       D(3, ("Reload Button Press event\n"));
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if (winwid != NULL)
          feh_reload_image(winwid, 0);
    }
    else if (ev->xbutton.button == 4 /* this is bad */ )
    {
       D(3, ("Button 4 Press event\n"));
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if ((winwid != NULL) && (winwid->type == WIN_TYPE_SLIDESHOW))
          slideshow_change_image(winwid, SLIDE_PREV);
    }
    else if (ev->xbutton.button == 5 /* this is bad */ )
    {
       D(3, ("Button 5 Press event\n"));
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if ((winwid != NULL) && (winwid->type == WIN_TYPE_SLIDESHOW))
          slideshow_change_image(winwid, SLIDE_NEXT);
    }
@@ -259,10 +215,9 @@ feh_event_handle_ButtonRelease(XEvent * ev)
    {
       /* if menus are open, close them, and execute action if needed */
 
-      if (ev->xbutton.window == menu_cover)
+      if (ev->xbutton.window == menu_cover) {
          feh_menu_hide(menu_root, True);
-      else if (menu_root)
-      {
+      } else if (menu_root) {
          feh_menu *m;
 
          if ((m = feh_menu_get_from_window(ev->xbutton.window)))
@@ -270,19 +225,16 @@ feh_event_handle_ButtonRelease(XEvent * ev)
             feh_menu_item *i = NULL;
 
             i = feh_menu_find_selected(m);
-            /* watch out for this. I put it this way around so the menu
-               goes away *before* we perform the action, if we start
-               freeing menus on hiding, it will break ;-) */
-            feh_menu_hide(menu_root, False);
-            feh_main_iteration(0);
-            if ((i) && (i->func))
-               (i->func) (m, i, i->data);
-            if(m->func_free)
-               m->func_free(m, m->data);
+            feh_menu_item_activate(m, i);
          }
       }
       D_RETURN_(4);
    }
+   
+   winwid = winwidget_get_from_window(ev->xbutton.window);
+   if (winwid && winwid->caption_entry) {
+     D_RETURN_(4);
+   }
 
    if ((ev->xbutton.button == opt.menu_button)
        &&
@@ -293,7 +245,6 @@ feh_event_handle_ButtonRelease(XEvent * ev)
       winwidget_destroy_all();
    else if (ev->xbutton.button == opt.next_button)
    {
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if (opt.mode == MODE_PAN)
       {
          if (winwid != NULL)
@@ -340,7 +291,7 @@ feh_event_handle_ButtonRelease(XEvent * ev)
                if (!thumbwin)
                {
                   thumbwin =
-                     winwidget_create_from_file(feh_list_add_front
+                     winwidget_create_from_file(gib_list_add_front
                                                 (NULL, thumbfile), s,
                                                 WIN_TYPE_THUMBNAIL_VIEWER);
                   winwidget_show(thumbwin);
@@ -348,7 +299,7 @@ feh_event_handle_ButtonRelease(XEvent * ev)
                else if (FEH_FILE(thumbwin->file->data) != thumbfile)
                {
                   free(thumbwin->file);
-                  thumbwin->file = feh_list_add_front(NULL, thumbfile);
+                  thumbwin->file = gib_list_add_front(NULL, thumbfile);
                   winwidget_rename(thumbwin, s);
                   feh_reload_image(thumbwin, 1);
                }
@@ -360,7 +311,6 @@ feh_event_handle_ButtonRelease(XEvent * ev)
             || (ev->xbutton.button == opt.zoom_button))
    {
       D(3, ("Mode-based Button Release event\n"));
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if (winwid != NULL)
       {
          D(3, ("Disabling mode\n"));
@@ -374,7 +324,6 @@ feh_event_handle_ButtonRelease(XEvent * ev)
             && ((opt.no_blur_ctrl_mask) || (ev->xbutton.state & ControlMask)))
    {
       D(3, ("Blur Button Release event\n"));
-      winwid = winwidget_get_from_window(ev->xbutton.window);
       if (winwid != NULL)
       {
          D(3, ("Disabling Blur mode\n"));
@@ -460,6 +409,16 @@ feh_event_handle_MotionNotify(XEvent * ev)
 {
    winwidget winwid = NULL;
    int dx, dy;
+   int scr_width, scr_height;
+
+   scr_width = scr->width;
+   scr_height = scr->height;
+#ifdef HAVE_LIBXINERAMA
+    if (opt.xinerama && xinerama_screens) {
+      scr_width = xinerama_screens[xinerama_screen].width;
+      scr_height = xinerama_screens[xinerama_screen].height;
+    }
+#endif /* HAVE_LIBXINERAMA */
 
    D_ENTER(5);
    if (menu_root)
@@ -495,12 +454,16 @@ feh_event_handle_MotionNotify(XEvent * ev)
           * screen. If so, and if the menu we are currently over is partially
           * hidden, slide the menu to the left and/or up until it is
           * fully visible */
+
+         /* FIXME: get this working nicely with xinerama screen edges --
+          * at the moment it does really funky stuff with
+          * scr_{width,height} instead of scr->{width,height} -- pabs*/
          if (mouseover_item
              && ((scr->width - (ev->xmotion.x + m->x)) < m->w
                  || (scr->height - (ev->xmotion.y + m->y)) < m->w))
          {
-            dx = scr->width - (m->x + m->w);
-            dy = scr->height - (m->y + m->h);
+            dx = scr_width - (m->x + m->w);
+            dy = scr_height - (m->y + m->h);
             dx = dx < 0 ? dx : 0;
             dy = dy < 0 ? dy : 0;
             if (dx || dy)
@@ -591,10 +554,10 @@ feh_event_handle_MotionNotify(XEvent * ev)
          {
             Imlib_Image temp;
 
-            temp = feh_imlib_create_rotated_image(winwid->im, 0.0);
-            winwid->im_w = feh_imlib_image_get_width(temp);
-            winwid->im_h = feh_imlib_image_get_height(temp);
-            feh_imlib_free_image_and_decache(temp);
+            temp = gib_imlib_create_rotated_image(winwid->im, 0.0);
+            winwid->im_w = gib_imlib_image_get_width(temp);
+            winwid->im_h = gib_imlib_image_get_height(temp);
+            gib_imlib_free_image_and_decache(temp);
             if (!winwid->full_screen && !opt.geom)
                winwidget_resize(winwid, winwid->im_w, winwid->im_h);
             winwid->has_rotated = 1;
@@ -618,17 +581,17 @@ feh_event_handle_MotionNotify(XEvent * ev)
 
          D(5, ("Blurring\n"));
 
-         temp = feh_imlib_clone_image(winwid->im);
+         temp = gib_imlib_clone_image(winwid->im);
          blur_radius = (((double) ev->xmotion.x / winwid->w) * 20) - 10;
          D(5, ("angle: %d\n", blur_radius));
          if (blur_radius > 0)
-            feh_imlib_image_sharpen(temp, blur_radius);
+            gib_imlib_image_sharpen(temp, blur_radius);
          else
-            feh_imlib_image_blur(temp, 0 - blur_radius);
+            gib_imlib_image_blur(temp, 0 - blur_radius);
          ptr = winwid->im;
          winwid->im = temp;
          winwidget_render_image(winwid, 0, 0);
-         feh_imlib_free_image_and_decache(winwid->im);
+         gib_imlib_free_image_and_decache(winwid->im);
          winwid->im = ptr;
       }
    }
@@ -647,12 +610,12 @@ feh_event_handle_MotionNotify(XEvent * ev)
             x = ev->xmotion.x - winwid->im_x;
             y = ev->xmotion.y - winwid->im_y;
             orig_im = winwid->im;
-            winwid->im = feh_imlib_clone_image(orig_im);
+            winwid->im = gib_imlib_clone_image(orig_im);
             imlib_context_set_image(winwid->im);
             imlib_apply_filter("bump_map_point(x=[],y=[],map=" PREFIX
                                "/share/feh/images/about.png);", &x, &y);
             winwidget_render_image(winwid, 0, 1);
-            feh_imlib_free_image_and_decache(winwid->im);
+            gib_imlib_free_image_and_decache(winwid->im);
             winwid->im = orig_im;
          }
          else if (winwid->type == WIN_TYPE_THUMBNAIL)
@@ -671,27 +634,27 @@ feh_event_handle_MotionNotify(XEvent * ev)
                   Imlib_Image origwin;
 
                   origwin = winwid->im;
-                  winwid->im = feh_imlib_clone_image(origwin);
-                  feh_imlib_image_fill_rectangle(winwid->im, thumbnail->x,
+                  winwid->im = gib_imlib_clone_image(origwin);
+                  gib_imlib_image_fill_rectangle(winwid->im, thumbnail->x,
                                                  thumbnail->y, thumbnail->w,
                                                  thumbnail->h, 50, 50, 255,
                                                  100);
-                  feh_imlib_image_draw_rectangle(winwid->im, thumbnail->x,
+                  gib_imlib_image_draw_rectangle(winwid->im, thumbnail->x,
                                                  thumbnail->y, thumbnail->w,
                                                  thumbnail->h, 255, 255, 255,
                                                  255);
-                  feh_imlib_image_draw_rectangle(winwid->im, thumbnail->x + 1,
+                  gib_imlib_image_draw_rectangle(winwid->im, thumbnail->x + 1,
                                                  thumbnail->y + 1,
                                                  thumbnail->w - 2,
                                                  thumbnail->h - 2, 0, 0, 0,
                                                  255);
-                  feh_imlib_image_draw_rectangle(winwid->im, thumbnail->x + 2,
+                  gib_imlib_image_draw_rectangle(winwid->im, thumbnail->x + 2,
                                                  thumbnail->y + 2,
                                                  thumbnail->w - 4,
                                                  thumbnail->h - 4, 255, 255,
                                                  255, 255);
                   winwidget_render_image(winwid, 0, 1);
-                  feh_imlib_free_image_and_decache(winwid->im);
+                  gib_imlib_free_image_and_decache(winwid->im);
                   winwid->im = origwin;
                }
                else
