@@ -1,6 +1,6 @@
 /* imlib.c
 
-Copyright (C) 1999,2000 Tom Gilbert.
+Copyright (C) 1999-2003 Tom Gilbert.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -24,7 +24,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
 #include "feh.h"
-#include "feh_list.h"
 #include "filelist.h"
 #include "winwidget.h"
 #include "options.h"
@@ -43,7 +42,26 @@ int depth;
 Atom wmDeleteWindow;
 XContext xid_context = 0;
 Window root = 0;
-winwidget progwin = NULL;
+
+/* Xinerama support */
+#ifdef HAVE_LIBXINERAMA
+XineramaScreenInfo *xinerama_screens = NULL;
+int xinerama_screen;
+int num_xinerama_screens;
+#endif /* HAVE_LIBXINERAMA */
+
+#ifdef HAVE_LIBXINERAMA
+void
+init_xinerama(void)
+{
+  if (opt.xinerama && XineramaIsActive(disp)) {
+    int major, minor;
+    xinerama_screen = 0;
+    XineramaQueryVersion(disp, &major, &minor);
+    xinerama_screens = XineramaQueryScreens(disp, &num_xinerama_screens);
+  }
+}
+#endif /* HAVE_LIBXINERAMA */
 
 void
 init_x_and_imlib(void)
@@ -60,6 +78,10 @@ init_x_and_imlib(void)
    scr = ScreenOfDisplay(disp, DefaultScreen(disp));
    xid_context = XUniqueContext();
 
+#ifdef HAVE_LIBXINERAMA
+   init_xinerama();
+#endif /* HAVE_LIBXINERAMA */
+
    imlib_context_set_display(disp);
    imlib_context_set_visual(vis);
    imlib_context_set_colormap(cm);
@@ -104,8 +126,9 @@ feh_load_image(Imlib_Image * im, feh_file * file)
       D_RETURN(4, 0);
 
    /* Handle URLs */
-   if ((!strncmp(file->filename, "http://", 7))
-       || (!strncmp(file->filename, "ftp://", 6)))
+   if ((!strncmp(file->filename, "http://", 7)) ||
+       (!strncmp(file->filename, "https://", 8)) ||
+       (!strncmp(file->filename, "ftp://", 6)))
    {
       char *tmpname = NULL;
       char *tempcpy;
@@ -607,14 +630,17 @@ feh_draw_zoom(winwidget w)
    int tw = 0, th = 0;
    Imlib_Image im = NULL;
    char buf[100];
+   static DATA8 atab[256];
 
    D_ENTER(4);
 
    if (!w->im)
       D_RETURN_(4);
 
-   if (!fn)
-      fn = feh_imlib_load_font("20thcent/12");
+   if (!fn) {
+      fn = gib_imlib_load_font(DEFAULT_FONT);
+      memset(atab, 0, sizeof(atab));
+   }
 
    if (!fn)
    {
@@ -626,22 +652,25 @@ feh_draw_zoom(winwidget w)
             (int) (w->im_w * w->zoom), (int) (w->im_h * w->zoom));
 
    /* Work out how high the font is */
-   feh_imlib_get_text_size(fn, buf, &tw, &th, IMLIB_TEXT_TO_RIGHT);
+   gib_imlib_get_text_size(fn, buf, NULL, &tw, &th, IMLIB_TEXT_TO_RIGHT);
 
-   tw += 2;
-   th += 2;
+   tw += 3;
+   th += 3;
    im = imlib_create_image(tw, th);
    if (!im)
       eprintf("Couldn't create image. Out of memory?");
 
-   feh_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 255);
+   gib_imlib_image_set_has_alpha(im, 1);
+   gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, th,
+                                               NULL, NULL, NULL, atab);
+   gib_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 0);
 
-   feh_imlib_text_draw(im, fn, 1, 1, buf, IMLIB_TEXT_TO_RIGHT, 255, 255, 255,
-                       255);
-
-   feh_imlib_render_image_on_drawable(w->bg_pmap, im, 0, w->h - th, 1, 0, 0);
-
-   feh_imlib_free_image_and_decache(im);
+   gib_imlib_text_draw(im, fn, NULL, 2, 2, buf, IMLIB_TEXT_TO_RIGHT,
+                       0, 0, 0, 255);
+   gib_imlib_text_draw(im, fn, NULL, 1, 1, buf, IMLIB_TEXT_TO_RIGHT,
+                       255, 255, 255, 255);
+   gib_imlib_render_image_on_drawable(w->bg_pmap, im, 0, w->h - th, 1, 1, 0);
+   gib_imlib_free_image_and_decache(im);
    D_RETURN_(4);
 }
 
@@ -651,6 +680,7 @@ feh_draw_filename(winwidget w)
    static Imlib_Font fn = NULL;
    int tw = 0, th = 0;
    Imlib_Image im = NULL;
+   static DATA8 atab[256];
 
    D_ENTER(4);
 
@@ -660,10 +690,11 @@ feh_draw_filename(winwidget w)
 
    if (!fn)
    {
+      memset(atab, 0, sizeof(atab));
       if (w->full_screen)
-         fn = feh_imlib_load_font("20thcent/16");
+         fn = gib_imlib_load_font(DEFAULT_FONT_BIG);
       else
-         fn = feh_imlib_load_font("20thcent/10");
+         fn = gib_imlib_load_font(DEFAULT_FONT);
    }
 
    if (!fn)
@@ -673,26 +704,184 @@ feh_draw_filename(winwidget w)
    }
 
    /* Work out how high the font is */
-   feh_imlib_get_text_size(fn, FEH_FILE(w->file->data)->filename, &tw, &th,
+   gib_imlib_get_text_size(fn, FEH_FILE(w->file->data)->filename, NULL, &tw, &th,
                            IMLIB_TEXT_TO_RIGHT);
 
-   tw += 2;
-   th += 2;
+   tw += 3;
+   th += 3;
    im = imlib_create_image(tw, th);
    if (!im)
       eprintf("Couldn't create image. Out of memory?");
 
-   feh_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 255);
+   gib_imlib_image_set_has_alpha(im, 1);
+   gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, th,
+                                               NULL, NULL, NULL, atab);
+   gib_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 0);
 
-   feh_imlib_text_draw(im, fn, 1, 1, FEH_FILE(w->file->data)->filename,
+   gib_imlib_text_draw(im, fn, NULL, 2, 2, FEH_FILE(w->file->data)->filename,
+                       IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
+   gib_imlib_text_draw(im, fn, NULL, 1, 1, FEH_FILE(w->file->data)->filename,
                        IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
 
-   feh_imlib_render_image_on_drawable(w->bg_pmap, im, 0, 0, 1, 0, 0);
+   gib_imlib_render_image_on_drawable(w->bg_pmap, im, 0, 0, 1, 1, 0);
 
-   feh_imlib_free_image_and_decache(im);
+   gib_imlib_free_image_and_decache(im);
    D_RETURN_(4);
 }
 
+char *build_caption_filename(feh_file *file) {
+  char *caption_filename;
+  char *s, *dir;
+  s = strrchr(file->filename, '/');
+  if (s) {
+    dir = estrdup(file->filename);
+    s = strrchr(dir, '/');
+    *s = '\0';
+  } else {
+    dir = estrdup(".");
+  }
+  caption_filename = estrjoin("",
+                              dir,
+                              "/",
+                              opt.caption_path,
+                              "/",
+                              file->name,
+                              ".txt",
+                              NULL);
+  free(dir);
+  return caption_filename;
+}
+
+void
+feh_draw_caption(winwidget w)
+{
+   static Imlib_Font fn = NULL;
+   int tw = 0, th = 0, ww, hh;
+   int x, y;
+   Imlib_Image im = NULL;
+   static DATA8 atab[256];
+   struct stat st;
+   char *p;
+   gib_list *lines, *l;
+   static gib_style *caption_style = NULL;
+   feh_file *file;
+
+   D_ENTER(4);
+
+   if (!w->file) {
+     D_RETURN_(4);
+   }
+   file = FEH_FILE(w->file->data);
+   if (!file->filename) {
+     D_RETURN_(4);
+   }
+
+   if (!file->caption) {
+      char *caption_filename;
+      caption_filename = build_caption_filename(file);
+      /* read caption from file */
+      file->caption = ereadfile(caption_filename);
+      free(caption_filename);
+   }
+
+   if (file->caption == NULL) {
+     /* caption file is not there, we want to cache that, otherwise we'll stat
+      * the damn file every time we render the image. Reloading an image will
+      * always cause the caption to be reread though so we're safe to do so.
+      * (Before this bit was added, when zooming a captionless image with
+      * captions enabled, the captions file would be stat()d like 30 times a
+      * second) - don't forget this function is called from
+      * winwidget_render_image().
+      */
+     file->caption = estrdup("");
+     D_RETURN_(4);
+   }
+
+   if (file->caption == '\0') {
+     D_RETURN_(4);
+   }
+
+   if (!caption_style) {
+     caption_style = gib_style_new("caption");
+     caption_style->bits = gib_list_add_front(caption_style->bits,
+                                            gib_style_bit_new(0,0,0,0,0,0));
+     caption_style->bits = gib_list_add_front(caption_style->bits, 
+                                            gib_style_bit_new(1,1,0,0,0,255));
+   }
+
+   if (!fn)
+   {
+      memset(atab, 0, sizeof(atab));
+      if (w->full_screen)
+         fn = gib_imlib_load_font(DEFAULT_FONT_BIG);
+      else
+         fn = gib_imlib_load_font(DEFAULT_FONT);
+   }
+
+   if (!fn)
+   {
+      weprintf("Couldn't load font for caption printing");
+      D_RETURN_(4);
+   }
+
+   lines = feh_wrap_string(file->caption, w->w, w->h, fn, NULL);
+   if (!lines)
+     D_RETURN_(4);
+
+   /* Work out how high/wide the caption is */
+   l = lines;
+   while (l) {
+     p = (char *) l->data;
+     gib_imlib_get_text_size(fn, p, caption_style, &ww, &hh, IMLIB_TEXT_TO_RIGHT);
+     if (ww > tw)
+       tw = ww;
+     th += hh;
+     if (l->next)
+       th += 1; /* line spacing */
+     l = l->next;
+   }
+   
+   /* we don't want the caption overlay larger than our window */
+   if (th > w->h)
+     th = w->h;
+   if (tw > w->w)
+     tw = w->w;
+
+   im = imlib_create_image(tw, th);
+   if (!im)
+      eprintf("Couldn't create image. Out of memory?");
+
+   gib_imlib_image_set_has_alpha(im, 1);
+   gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, th,
+                                               NULL, NULL, NULL, atab);
+   gib_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 0);
+   
+   l = lines;
+   x = 0;
+   y = 0;
+   while (l) {
+     p = (char *) l->data;
+     gib_imlib_get_text_size(fn, p, caption_style, &ww, &hh, IMLIB_TEXT_TO_RIGHT);
+     x = (tw - ww) / 2;
+     if (w->caption_entry) {
+     gib_imlib_text_draw(im, fn, caption_style, x, y, p, IMLIB_TEXT_TO_RIGHT,
+                         255, 255, 0, 255);
+     } else {
+     gib_imlib_text_draw(im, fn, caption_style, x, y, p, IMLIB_TEXT_TO_RIGHT,
+                         255, 255, 255, 255);
+     }
+                
+     y += hh + 1; /* line spacing */
+     l = l->next;
+   }
+   
+   gib_imlib_render_image_on_drawable(w->bg_pmap, im, (w->w - tw) / 2, w->h - th, 1, 1, 0);
+   gib_imlib_free_image_and_decache(im);
+   gib_list_free_and_data(lines);
+   D_RETURN_(4);
+}
+
+
 unsigned char reset_output = 0;
 
 void
@@ -707,7 +896,7 @@ feh_display_status(char stat)
    D(5, ("filelist %p, filelist->next %p\n", filelist, filelist->next));
 
    if (!init_len)
-      init_len = feh_list_length(filelist);
+      init_len = gib_list_length(filelist);
 
    if (i)
    {
@@ -723,7 +912,7 @@ feh_display_status(char stat)
          int len;
          char buf[50];
 
-         len = feh_list_length(filelist);
+         len = gib_list_length(filelist);
          snprintf(buf, sizeof(buf), " %5d/%d (%d)\n[%3d%%] ", i, init_len,
                   len, ((int) ((float) i / init_len * 100)));
          fprintf(stdout, buf);
@@ -753,14 +942,117 @@ void feh_edit_inplace_orient(winwidget w, int orientation) {
 
   ret = feh_load_image(&old, FEH_FILE(w->file->data));
   if(ret) {
-    feh_imlib_image_orientate(old, orientation);
-    feh_imlib_save_image(old, FEH_FILE(w->file->data)->filename);
-    feh_imlib_free_image(old);
+    gib_imlib_image_orientate(old, orientation);
+    gib_imlib_save_image(old, FEH_FILE(w->file->data)->filename);
+    gib_imlib_free_image(old);
     feh_reload_image(w, 1);
   } else {
-    wprintf("failed to load image from disk to edit it in place\n");
+    weprintf("failed to load image from disk to edit it in place\n");
   }
   
   D_RETURN_(4);
 }
 
+
+/* TODO max_height is ignored... Could use a function which generates a
+ * transparent text overlay image, with wrapping and all. Would be useful */
+gib_list *
+feh_wrap_string(char *text, int wrap_width, int max_height, Imlib_Font fn, gib_style * style)
+{
+   gib_list *ll, *lines = NULL, *list = NULL, *words;
+   gib_list *l = NULL;
+   char delim[2] = { '\n', '\0' };
+   int w, line_width;
+   int tw, th;
+   char *p, *pp;
+   char *line = NULL;
+   char *temp;
+   int space_width = 0, m_width = 0, t_width = 0, new_width = 0;
+
+   lines = gib_string_split(text, delim);
+
+   if (wrap_width)
+   {
+      gib_imlib_get_text_size(fn, "M M", style, &t_width, NULL,
+                              IMLIB_TEXT_TO_RIGHT);
+      gib_imlib_get_text_size(fn, "M", style, &m_width, NULL,
+                              IMLIB_TEXT_TO_RIGHT);
+      space_width = t_width - (2 * m_width);
+      w = wrap_width;
+      l = lines;
+      while (l)
+      {
+         line_width = 0;
+         p = (char *) l->data;
+         /* quick check to see if whole line fits okay */
+         gib_imlib_get_text_size(fn, p, style, &tw, &th, IMLIB_TEXT_TO_RIGHT);
+         if (tw <= w) {
+            list = gib_list_add_end(list, estrdup(p));
+         } else if (strlen(p) == 0) {
+            list = gib_list_add_end(list, estrdup(""));
+         } else if (!strcmp(p, " ")) {
+            list = gib_list_add_end(list, estrdup(" "));
+         } else {
+            words = gib_string_split(p, " ");
+            if (words) {
+               ll = words;
+               while (ll) {
+                  pp = (char *) ll->data;
+                  if (strcmp(pp, " ")) {
+                     gib_imlib_get_text_size(fn, pp, style, &tw, &th,
+                                             IMLIB_TEXT_TO_RIGHT);
+                     if (line_width == 0)
+                        new_width = tw;
+                     else
+                        new_width = line_width + space_width + tw;
+                     if (new_width <= w) {
+                        /* add word to line */
+                        if (line) {
+                           int len;
+
+                           len = strlen(line) + strlen(pp) + 2;
+                           temp = emalloc(len);
+                           snprintf(temp, len, "%s %s", line, pp);
+                           free(line);
+                           line = temp;
+                        } else {
+                          line = estrdup(pp);
+                        }
+                        line_width = new_width;
+                     } else if (line_width == 0) {
+                        /* can't fit single word in :/
+                           increase width limit to width of word
+                           and jam the bastard in anyhow */
+                        w = tw;
+                        line = estrdup(pp);
+                        line_width = new_width;
+                     } else {
+                        /* finish this line, start next and add word there */
+                        if (line) {
+                           list = gib_list_add_end(list, estrdup(line));
+                           free(line);
+                           line = NULL;
+                        }
+                        line = estrdup(pp);
+                        line_width = tw;
+                     }
+                  }
+                  ll = ll->next;
+               }
+               if (line) {
+                  /* finish last line */
+                  list = gib_list_add_end(list, estrdup(line));
+                  free(line);
+                  line = NULL;
+                  line_width = 0;
+               }
+               gib_list_free_and_data(words);
+            }
+         }
+         l = l->next;
+      }
+      gib_list_free_and_data(lines);
+      lines = list;
+   }
+   return lines;
+}
