@@ -1,6 +1,6 @@
 /* options.c
 
-Copyright (C) 1999,2000 Tom Gilbert.
+Copyright (C) 1999-2003 Tom Gilbert.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -24,7 +24,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
 #include "feh.h"
-#include "feh_list.h"
 #include "filelist.h"
 #include "options.h"
 
@@ -54,9 +53,12 @@ init_parse_options(int argc, char **argv)
    opt.slideshow_delay = -1.0;
    opt.thumb_w = 60;
    opt.thumb_h = 60;
-   opt.menu_font = estrdup("20thcent/12");
-   opt.font = estrdup("20thcent/8");
-   opt.menu_bg = estrdup(PREFIX "/share/feh/images/menubg.png");
+   opt.menu_font = estrdup(DEFAULT_MENU_FONT);
+   opt.font = estrdup(DEFAULT_FONT);
+   opt.menu_bg = estrdup(PREFIX "/share/feh/images/menubg_default.png");
+   opt.menu_style = estrdup(PREFIX "/share/feh/fonts/menu.style");
+   opt.menu_border = 4;
+
    opt.next_button = 1;
    opt.zoom_button = 2;
    opt.menu_button = 3;
@@ -68,8 +70,17 @@ init_parse_options(int argc, char **argv)
    opt.blur_button = 1;
    opt.no_blur_ctrl_mask = 0;
 
+   opt.no_jump_on_resort = 0;
+
    opt.builtin_http = 0;
 
+   opt.xinerama = 0;
+   opt.screen_clip = 1;
+#ifdef HAVE_LIBXINERAMA
+   /* if we're using xinerama, then enable it by default */
+   opt.xinerama = 1;
+#endif /* HAVE_LIBXINERAMA */
+   
    D(3, ("About to parse env options (if any)\n"));
    /* Check for and parse any options in FEH_OPTIONS */
    feh_parse_environment_options();
@@ -88,7 +99,7 @@ init_parse_options(int argc, char **argv)
          here, as files specified on the commandline end up at the *end* of
          the combined filelist, in the specified order. */
       D(3, ("About to load filelist from file\n"));
-      filelist = feh_list_cat(filelist, feh_read_filelist(opt.filelistfile));
+      filelist = gib_list_cat(filelist, feh_read_filelist(opt.filelistfile));
    }
 
    D(4, ("Options parsed\n"));
@@ -96,7 +107,7 @@ init_parse_options(int argc, char **argv)
    if(opt.bgmode)
       D_RETURN_(4);
 
-   if (feh_list_length(filelist) == 0)
+   if (gib_list_length(filelist) == 0)
       show_mini_usage();
 
    check_options();
@@ -327,7 +338,7 @@ feh_parse_option_array(int argc, char **argv)
       {"full-screen", 0, 0, 'F'},
       {"auto-zoom", 0, 0, 'Z'},
       {"ignore-aspect", 0, 0, 'X'},
-      {"draw_filename", 0, 0, 'd'},
+      {"draw-filename", 0, 0, 'd'},
       {"preload", 0, 0, 'p'},
       {"reverse", 0, 0, 'n'},
       {"thumbnails", 0, 0, 't'},
@@ -375,6 +386,13 @@ feh_parse_option_array(int argc, char **argv)
       {"bg-center", 1, 0, 201},
       {"bg-scale", 1, 0, 202},
       {"bg-seamless", 1, 0, 203},
+      {"menu-style", 1, 0, 204},
+      {"zoom", 1, 0, 205},
+      {"xinerama", 1, 0, 206},
+      {"screen-clip", 1, 0, 207},
+      {"menu-border", 1, 0, 208},
+      {"caption-path", 1, 0, 209},
+      {"no-jump-on-resort",0,0,210},
       {0, 0, 0, 0}
    };
    int optch = 0, cmdx = 0, i = 0;
@@ -639,6 +657,28 @@ feh_parse_option_array(int argc, char **argv)
            opt.bgmode = BG_MODE_SEAMLESS;
            opt.output_file = estrdup(optarg);
            break;
+        case 204:
+           free(opt.menu_style);
+           opt.menu_style = estrdup(optarg);
+           break;
+        case 205:
+           opt.default_zoom = atoi(optarg);
+           break;
+        case 206:
+           opt.xinerama = atoi(optarg);
+           break;
+        case 207:
+           opt.screen_clip = atoi(optarg);
+           break;
+        case 208:
+           opt.menu_border = atoi(optarg);
+           break; 
+        case 209:
+           opt.caption_path = estrdup(optarg);
+           break; 
+	case 210:
+           opt.no_jump_on_resort = 1;
+           break;
         default:
            break;
       }
@@ -754,6 +794,8 @@ show_usage(void)
            "                            the content of those directories. (Take it easy)\n"
            "  -z, --randomize           When viewing multiple files in a slideshow,\n"
            "                            randomise the file list before displaying\n"
+           "  --no-jump-on-resort       Don't jump to the first image when the filelist\n"
+           "                            is resorted.\n"
            "  -g, --geometry STRING     Limit (and don't change) the window size. Takes\n"
            "                            an X-style geometry string like 640x480.\n"
            "                            Note that larger images will be zoomed out to fit\n"
@@ -783,6 +825,10 @@ show_usage(void)
            "  -F, --full-screen         Make the window fullscreen\n"
            "  -Z, --auto-zoom           Zoom picture to screen size in fullscreen mode,\n"
            "                            is affected by the option --stretch\n"
+           "      --zoom PERCENT        Zooms images by a PERCENT, when in full screen\n"
+           "                            mode or when window geometry is fixed. If combined\n"
+           "                            with --auto-zoom, zooming will be limited to the\n"
+           "                            the size.\n"
            "  -w, --multiwindow         Disable slideshow mode. With this setting,\n"
            "                            instead of opening multiple files in slideshow\n"
            "                            mode, multiple windows will be opened.\n"
@@ -797,14 +843,21 @@ show_usage(void)
            "                            via http, or even on your local machine.\n"
            "  -Q, --builtin             Use builtin http grabber to grab remote files\n"
            "                            instead of wget.\n"
-           "                            mechanism, useful if you need to use a proxy or\n"
-           "                            something.\n"
+           "                            mechanism, useful if don't have wget.\n"
            "  -k, --keep-http           When viewing files using http, feh normally\n"
            "                            deletes the local copies after viewing, or,\n"
            "                            if caching, on exit. This option prevents this\n"
            "                            so that you get to keep the local copies.\n"
            "                            They will be in the current working directory\n"
            "                            with \"feh\" in the name.\n"
+           "      --caption-path PATH   Path to directory containing image captions.\n"
+           "                            This turns on caption viewing, and if captions\n"
+           "                            are found in PATH, which is relative to the\n"
+           "                            directory of each image, they are overlayed\n"
+           "                            on the displayed image.\n"
+           "                            e.g with caption path \"captions\", and viewing\n"
+           "                            image images/foo.jpg, caption will be looked for\n"
+           "                            as \"images/captions/foo.jpg.txt\"\n"
            "  -j, --output-dir          Output directory for saved files.  Really only\n"
            "                            useful with the -k flag.\n"
            "  -G, --wget-timestamp      When viewing http images with reload set (eg\n"
@@ -837,7 +890,7 @@ show_usage(void)
            "                            The action will be executed by /bin/sh. Use\n"
            "                            format specifiers to refer to image info. See\n"
            "                            FORMAT SPECIFIERS for examples\n"
-           "                            Eg. -X \"mv %%f ~/images/%%n\"\n"
+           "                            Eg. -A \"mv %%f ~/images/%%n\"\n"
            "                            In slideshow mode, the next image will be shown\n"
            "                            after running the command, in multiwindow mode,\n"
            "                            the window will be closed.\n"
@@ -875,7 +928,11 @@ show_usage(void)
            "      --fontpath PATH       Specify an extra directory to look in for fonts,\n"
            "                            can be used multiple times to add multiple paths.\n"
            "  -M, --menu-font FONT      Use FONT for the font in menus.\n"
+           "      --menu-style FILE     Use FILE as the style descriptor for menu text.\n"
            "      --menu-bg BG          Use BG for the background image in menus.\n"
+           "      --menu-border INT     Specify number of pixels that define the menu\n"
+           "                            background's border. Borders are not stretched\n"
+           "                            when images are scaled.\n"
            "  -N, --no-menus            Don't load or show any menus.\n"
            "  -1, --next-button B       Use button B to advance to the next image in any\n"
            "                            mode (defaults to 1, usually the left button).\n"
@@ -895,6 +952,12 @@ show_usage(void)
            "                            any mode (default=1).\n"
            "  -9, --no-blur-ctrl-mask   Don't require CTRL+Button for blurring in\n"
            "                            any mode -- just use the button (default=off).\n"
+           "      --xinerama [0|1]      Enable/disable Xinerama support.  Has no effect\n"
+           "                            unless you have an Xinerama compiled in.\n"
+           "      --screen-clip [0|1]   Enable/disable window clipping based on screen\n"
+           "                            size.  WARNING: with this option disabled,\n"
+           "                            image windows could become very large, making\n"
+           "                            them unmanageable in certain window managers.\n."
            " FORMAT SPECIFIERS\n"
            "                            %%f image path/filename\n"
            "                            %%n image name\n"
@@ -947,7 +1010,7 @@ show_usage(void)
            "  -e FONT                   Use FONT to print the information under each\n"
            "                            thumbnail. FONT should be defined in the form\n"
            "                            fontname/size(points). eg -e myfont/12\n"
-           "  -t,--title-font FONT      Use FONT to print a title on the index, if no\n"
+           "  -t, --title-font FONT     Use FONT to print a title on the index, if no\n"
            "                            font is specified, a title will not be printed\n"
            " SLIDESHOW KEYS\n"
            " The default mode for viewing mulitple images is Slideshow mode\n"
@@ -955,6 +1018,13 @@ show_usage(void)
            " p, P, <BACKSPACE>, <LEFT>  Goto previous slide\n"
            " n, N, <SPACE>, <RIGHT>     Goto next slide\n"
            " r, R                       Reload image (good for webcams)\n"
+           " v, V                       Toggle fullscreen\n"
+           " m, M                       Show popup menu\n"
+           " c, C                       Caption entry mode. If --caption-path has been\n"
+           "                            specified, then this enables caption editing.\n"
+           "                            The caption will turn yellow and be editable,\n"
+           "                            hit enter to confirm and save the caption, or\n"
+           "                            hit escape to cancel and revert the caption.\n"
            " w, W                       Size window to current image dimensions\n"
            " h, H                       Pause the slideshow (only useful when using\n"
            " s, S                       Save current image to unique filename\n"
@@ -963,13 +1033,20 @@ show_usage(void)
            " <, >                       In place editing, rotate 90 degrees right/left\n"
            " <HOME>                     Goto first slide\n"
            " <END>                      Goto last slide\n"
+           " <ESCAPE>                   Quit the slideshow\n"
            " +, =                       Increase reload delay\n"
            " -, _                       Decrease reload delay\n"
            " <DELETE>                   Remove the currently viewed file from the filelist\n"
            " <CTRL+DELETE>              Delete the currently viewed file and remove it\n"
            "                            from the filelist\n"
            " x, X                       Close current window\n"
-           " q, Q                       Quit the slideshow\n" "\n"
+           " q, Q                       Quit the slideshow\n"
+	         " <KEYPAD LEFT>              Move the image to the left\n"
+           " <KEYPAD RIGHT>             Move the image to the right\n"
+           " <KEYPAD +>                 Zoom in\n"
+           " <KEYPAD ->                 Zoom out\n"
+           " <KEYPAD *>                 Zoom to 100%\n"
+           " <KEYPAD />                 Zoom to fit the window\n" "\n"
            " MOUSE ACTIONS\n"
            " When viewing an image, a click of mouse button 1 moves to the next image\n"
            " (slideshow mode only), a drag of mouse button 1 pans the image, if the\n"
@@ -984,7 +1061,7 @@ show_usage(void)
            " options.\n" "\n" "See 'man feh' for more detailed information\n"
            "\n"
            "This program is free software see the file COPYING for licensing info.\n"
-           "Copyright Tom Gilbert (and various contributors) 1999, 2000\n"
+           "Copyright Tom Gilbert (and various contributors) 1999-2003\n"
            "Email bugs to <feh_sucks@linuxbrit.co.uk>\n");
    exit(0);
 }
@@ -1036,16 +1113,19 @@ feh_create_default_config(char *rcfile)
            "screensave --full-screen --randomize --slideshow-delay 5\n" "\n"
            "# Add <img> tags to your html with ease :-)\n"
            "newimg -q -L \"<img src=\\\"%%f\\\" alt=\\\"%%n\\\" border=\\\"0\\\" width=\\\"%%w\\\" height=\\\"%%h\\\">\"\n"
-           "\n" "# Different menus\n" "chrome --menu-bg " PREFIX
-           "/share/feh/images/menubg_chrome.png\n" "brushed --menu-bg " PREFIX
-           "/share/feh/images/menubg_brushed.png\n" "pastel --menu-bg " PREFIX
-           "/share/feh/images/menubg_pastel.png\n" "aluminium --menu-bg "
-           PREFIX "/share/feh/images/menubg_aluminium.png\n" "wood --menu-bg "
-           PREFIX "/share/feh/images/menubg_wood.png\n" "aqua --menu-bg "
-           PREFIX "/share/feh/images/menubg_aqua.png\n" "orange --menu-bg "
-           PREFIX "/share/feh/images/menubg_orange.png\n" "light --menu-bg "
-           PREFIX "/share/feh/images/menubg_light.png\n" "britney --menu-bg "
-           PREFIX "/share/feh/images/menubg_britney.png\n");
+           "\n"
+           "# Different menus\n"
+           "chrome --menu-bg " PREFIX" /share/feh/images/menubg_chrome.png\n"
+           "brushed --menu-bg " PREFIX "/share/feh/images/menubg_brushed.png\n"
+           "pastel --menu-bg " PREFIX "/share/feh/images/menubg_pastel.png\n"
+           "aluminium --menu-bg " PREFIX "/share/feh/images/menubg_aluminium.png\n"
+           "wood --menu-bg " PREFIX "/share/feh/images/menubg_wood.png\n"
+           "aqua --menu-bg " PREFIX "/share/feh/images/menubg_aqua.png\n"
+           "sky --menu-bg " PREFIX "/share/feh/images/menubg_sky.png\n"
+           "orange --menu-bg " PREFIX "/share/feh/images/menubg_orange.png\n"
+           "light --menu-bg " PREFIX "/share/feh/images/menubg_light.png\n"
+           "black --menu-bg " PREFIX "/share/feh/images/menubg_black.png --menu-style " PREFIX "/share/feh/fonts/black.style\n"
+           "britney --menu-bg " PREFIX "/share/feh/images/menubg_britney.png\n");
    fclose(fp);
 
    D_RETURN_(4);
